cvs_id = "$Id: Config.py,v 1.21 2003/10/25 13:33:36 juri Exp $"

import cPickle, os, traceback
from urlparse import urlparse
from error import log, logtb

import straw
import gconf
import gobject

GCONF_STRAW_ROOT = "/apps/straw"
GCONF_STRAW_POLL_FREQ = GCONF_STRAW_ROOT + "/general/poll_frequency"
GCONF_STRAW_ITEMS_STORED = GCONF_STRAW_ROOT + "/general/number_of_items_stored"
GCONF_STRAW_ITEM_ORDER = GCONF_STRAW_ROOT + "/general/item_order_newest"
GCONF_STRAW_OFFLINE = GCONF_STRAW_ROOT + "/general/offline"
GCONF_STRAW_LAST_POLL = GCONF_STRAW_ROOT + "/general/last_poll"
GCONF_STRAW_WINDOW_H = GCONF_STRAW_ROOT + "/ui/window_height"
GCONF_STRAW_WINDOW_W = GCONF_STRAW_ROOT + "/ui/window_width"
GCONF_STRAW_MAIN_PANE_POSITION = GCONF_STRAW_ROOT + "/ui/main_pane_position"
GCONF_STRAW_SUB_PANE_POSITION  = GCONF_STRAW_ROOT + "/ui/sub_pane_position"

def ensure_directory(strawdir):
    if os.path.exists(strawdir):
        if not os.path.isdir(strawdir):
            raise Exception
        return 0
    os.mkdir(strawdir)
    return 1

def config_to_gconf(cf):
    straw.config.poll_frequency = cf.get('poll_frequency')
    straw.config.number_of_items_stored = cf.get('number_of_items_stored')
    straw.config.item_order = cf.get('item_order')
    straw.config.main_window_size = cf.get('main_window_size')
    straw.config.main_pane_position = cf.get('main_pane_position')
    straw.config.sub_pane_position = cf.get('sub_pane_position')
    straw.config.offline = cf.get('offline')

    del cf['poll_frequency']
    del cf['number_of_items_stored']
    del cf['item_order']
    del cf['main_window_size']
    del cf['main_pane_position']
    del cf['sub_pane_position']
    del cf['offline']


class ConfigSaver:
    def __init__(self):
        self._config_file = os.path.join(straw.config._straw_dir, "config")
        # change to FeedsChangedSignal
        self.load_config()
        straw.config.signal_connect(straw.LocalConfigOptionSignal,self.notify_change)

    def notify_change(self, signal):
        self.save_config()
        return

    def load_config(self):
        if os.path.exists(self._config_file):
            fl = straw.FeedList()
            f = open(self._config_file)
            cf = cPickle.load(f)
            straw.config.feed_id_seq = cf.get('feed_id_seq',0)
            for feed in cf['feeds']:
                nf = straw.create_empty_feed()
                nf.undump(feed)
                fl.append(nf)
            straw.config.feeds = fl
            # migrate old config to gconf
            if cf.has_key('poll_frequency'):
                config_to_gconf(cf)

            f.close()
        return

    def save_config(self):
        cf = {}
        l = []
        for f in straw.config.feeds:
            l.append(f.dump())
        cf['feeds'] = l
        cf['feed_id_seq'] = straw.config.feed_id_seq
        pickle_file = open(self._config_file, 'w')
        cPickle.dump(cf, pickle_file, 1)
        pickle_file.close()
        return

class BaseProxyConfig(object):
    def get_config_working(self):
        return self._config_working

    config_working = property(get_config_working, None, None, "")

    def get_host(self):
        return self._host

    def set_host(self, host):
        self._host = host
        self._ip = None
        if self._host:
            straw.lookup_manager.lookup(self._host, self.ip_lookup_callback)

    host = property(get_host, set_host, None, "")

    def get_ip(self):
        return self._ip

    def set_ip(self, ip):
        assert self._ip is None, "Overwriting proxy IP not allowed!"
        if ip is None:
            self.use = 0
        self._ip = ip

    ip = property(get_ip, set_ip, None, "")

    def ip_lookup_callback(self, host, ip, data):
        self.ip = ip

    def _is_waiting(self):
        return self.use and not self._ip

    is_waiting = property(_is_waiting, None, None, "")

class GconfProxyConfig(BaseProxyConfig):
    """Encapsulate proxy use and location information (host, port, ip),
    gconf reading and name lookup logic"""

    GCONF_PROXY_ROOT = "/system/proxy"
    GCONF_PROXY_MODE = GCONF_PROXY_ROOT + "/mode"
    GCONF_HTTP_PROXY_ROOT = "/system/http_proxy"
    GCONF_HTTP_PROXY_HOST = GCONF_HTTP_PROXY_ROOT + "/host"
    GCONF_HTTP_PROXY_PORT = GCONF_HTTP_PROXY_ROOT + "/port"
    GCONF_HTTP_PROXY_USE_AUTHENTICATION = GCONF_HTTP_PROXY_ROOT + "/use_authentication"
    GCONF_HTTP_PROXY_USER = GCONF_HTTP_PROXY_ROOT + "/authentication_user"
    GCONF_HTTP_PROXY_PASSWORD = GCONF_HTTP_PROXY_ROOT + "/authentication_password"

    def __init__(self):
        client = gconf.client_get_default()
        self._config_working = client.dir_exists(self.GCONF_PROXY_ROOT) and client.dir_exists(self.GCONF_HTTP_PROXY_ROOT)
        if not self._config_working:
            return
        client.add_dir(self.GCONF_PROXY_ROOT, gconf.CLIENT_PRELOAD_RECURSIVE)
        client.add_dir(self.GCONF_HTTP_PROXY_ROOT, gconf.CLIENT_PRELOAD_RECURSIVE)
        client.notify_add(self.GCONF_PROXY_MODE, self.proxy_mode_changed)
        client.notify_add(self.GCONF_HTTP_PROXY_HOST, self.proxy_host_changed)
        client.notify_add(self.GCONF_HTTP_PROXY_PORT, self.proxy_port_changed)
        client.notify_add(self.GCONF_HTTP_PROXY_USE_AUTHENTICATION,
                          self.proxy_auth_changed)
        client.notify_add(self.GCONF_HTTP_PROXY_USER, self.proxy_auth_changed)
        client.notify_add(self.GCONF_HTTP_PROXY_PASSWORD, self.proxy_auth_changed)

        self._ip = None
        pm = client.get_string(self.GCONF_PROXY_MODE)
        self.use = pm is not None and pm != "none"
        self.host = client.get_string(self.GCONF_HTTP_PROXY_HOST)
        self.port = client.get_int(self.GCONF_HTTP_PROXY_PORT)
        self.use_authentication = client.get_bool(
            self.GCONF_HTTP_PROXY_USE_AUTHENTICATION)
        self.user = client.get_string(self.GCONF_HTTP_PROXY_USER)
        self.password = client.get_string(self.GCONF_HTTP_PROXY_PASSWORD)

    # here be gconf logic
    def proxy_mode_changed(self, client, notify_id, entry, *args):
        value = entry.value.get_string()
        self.use = value is not None and value != "none"

    def proxy_host_changed(self, client, notify_id, entry, *args):
        value = entry.value.get_string()
        if value is not None and len(value):
            self.host = value
        else:
            self.use = 0

    def proxy_port_changed(self, client, notify_id, entry, *args):
        value = entry.value.get_int()
        if value is not None and value > 0:
            self.port = value
        else:
            self.use = 0

    def proxy_auth_changed(self, client, notify_id, entry, *args):
        value = entry.value
        if entry.key == self.GCONF_HTTP_PROXY_USE_AUTHENTICATION:
            if not value:
                self.use_authentication = 0
            else:
                self.use_authentication = 1
        elif entry.key == self.GCONF_HTTP_PROXY_USER:
            self.user = value
        elif entry.key == self.GCONF_HTTP_PROXY_PASSWORD:
            self.password = value

class EnvironmentProxyConfig(BaseProxyConfig):
    """Encapsulate proxy use and location information, environment reading"""

    def __init__(self):
        try:
            result = self._read_env()
        except:
            result = 0
        self._config_working = result
        self.use = self._config_working

    def _read_env(self):
        proxy = os.getenv("http_proxy")
        if not proxy:
            return 0
        pparts = urlparse(proxy)
        proto = pparts[0]
        host = pparts[1]
        if proto != "http" or not len(host):
            return 0
        hparts = host.split('@')
        auth = self.user = self.password = self.use_authentication = None
        if len(hparts) > 2:
            return 0
        if len(hparts) == 2:
            auth, host = hparts
        if auth:
            aparts = auth.split(':')
            if len(aparts) != 2:
                return 0
            self.user, self.password = aparts
            self.use_authentication = 1
        self.host = host
        self.port = 80
        hparts = host.split(':')
        if len(hparts) > 2:
            return 0
        if len(hparts) == 2:
            self.host = hparts[0]
            self.port = int(hparts[1])
        return 1

class NullProxyConfig(BaseProxyConfig):
    def __init__(self):
        self.use = 0
        self._config_working = 0
        self._host = None
        self._ip = None
        self.port = None
        self.user = None
        self.password = None
        self.use_authentication = None

class Config(object, straw.SignalEmitter):
    def __init__(self):
        self.client = gconf.client_get_default()
        self.client.add_dir(GCONF_STRAW_ROOT, gconf.CLIENT_PRELOAD_ONELEVEL)
        straw.SignalEmitter.__init__(self)
        self.initialize_slots(straw.FeedsChangedSignal,
                              straw.ItemOrderChangedSignal,
                              straw.OfflineModeChangedSignal,
                              straw.LastPollChangedSignal)
        self._feed_id_seq = 0
        self._straw_dir = os.path.join(os.getenv("HOME"), ".straw")
        self._feeds = straw.FeedList()
        self._feeds.signal_connect(straw.FeedsChangedSignal,
                                   self.feeds_changed)
        pcchoices = (GconfProxyConfig, EnvironmentProxyConfig, NullProxyConfig)
        for p in pcchoices:
            self._proxy_config = p()
            if self._proxy_config.config_working:
                break
        self.first_time = ensure_directory(self._straw_dir)

        self._last_poll = self.client.get_int(GCONF_STRAW_LAST_POLL)

        if self._last_poll is None or self._last_poll == 0:
            self._load_defaults()
            return

        self._items_stored = self.client.get_int(GCONF_STRAW_ITEMS_STORED)
        self._item_order = self.client.get_bool(GCONF_STRAW_ITEM_ORDER)
        self._main_window_size = (self.client.get_int(GCONF_STRAW_WINDOW_W), self.client.get_int(GCONF_STRAW_WINDOW_H))
        self._main_pane_position = self.client.get_int(GCONF_STRAW_MAIN_PANE_POSITION)
        self._sub_pane_position = self.client.get_int(GCONF_STRAW_SUB_PANE_POSITION)
        self._offline = self.client.get_bool(GCONF_STRAW_OFFLINE)
        self._poll_freq = self.client.get_int(GCONF_STRAW_POLL_FREQ)

    def _load_defaults(self):
        self._poll_freq = 3600
        self._items_stored = 30
        self._item_order = bool(1)
        self._main_window_size = (680, 580)
        self._main_pane_position = 180
        self._sub_pane_position = 180
        self._offline = bool(1)
        self._last_poll = 0

    def get_proxy_config(self):
        return self._proxy_config

    proxy_config = property(get_proxy_config, None, None, "")

    # and the rest of the stuff
    def feeds_changed(self, signal):
        self.emit_signal(straw.FeedsChangedSignal(self))

    def set_feeds(self, feeds):
        self._feeds = straw.FeedList(feeds)
        self._feeds.signal_connect(straw.FeedsChangedSignal,
                                   self.feeds_changed)
        self.emit_signal(straw.FeedsChangedSignal(self))
        return

    def get_feeds(self):
        return self._feeds

    feeds = property(get_feeds, set_feeds, None, "Feeds")

    def set_poll_frequency(self, poll_frequency=60):
        if self._poll_freq != poll_frequency:
            self._poll_freq = poll_frequency
            self.client.set_int(GCONF_STRAW_POLL_FREQ, poll_frequency)
        return

    def get_poll_frequency(self):
        return self._poll_freq

    poll_frequency = property(get_poll_frequency, set_poll_frequency, None,
                              "Polling frequency")

    def set_last_poll(self, last_poll):
        if self._last_poll != last_poll:
            self._last_poll = last_poll
            self.client.set_int(GCONF_STRAW_LAST_POLL, last_poll)
            self.emit_signal(straw.LastPollChangedSignal(self))
        return

    def get_last_poll(self):
        return self._last_poll

    last_poll = property(get_last_poll, set_last_poll, None, "Last polled")

    def get_number_of_items_stored(self):
        return self._items_stored

    def set_number_of_items_stored(self, num=30):
        if self._items_stored != num:
            self._items_stored = num
            self.client.set_int(GCONF_STRAW_ITEMS_STORED, num)
            self.emit_signal(straw.NumberOfItemsStoredChangedSignal(self))
        return

    number_of_items_stored = property(get_number_of_items_stored,
                                      set_number_of_items_stored, None,
                                      "Number of items to store per feed")

    def set_item_order(self, order):
        if self._item_order != order:
            self._item_order = order
            self.client.set_bool(GCONF_STRAW_ITEM_ORDER, order)
            self.emit_signal(straw.ItemOrderChangedSignal(self))
        return

    def get_item_order(self):
        return self._item_order

    item_order = property(get_item_order, set_item_order, "Ordering of items")

    def get_feed_id_seq(self):
        return self._feed_id_seq

    def set_feed_id_seq(self, id):
        self._feed_id_seq = id
        return

    def next_feed_id_seq(self):
        self._feed_id_seq += 1
        return self._feed_id_seq

    feed_id_seq = property(get_feed_id_seq, set_feed_id_seq, None, "")

    def set_main_window_size(self, size):
        if self._main_window_size != size:
            self._main_window_size = size
            self.client.set_int(GCONF_STRAW_WINDOW_W,size[0])
            self.client.set_int(GCONF_STRAW_WINDOW_H,size[1])
            self.emit_signal(straw.MainWindowSizeChangedSignal(self))
        return

    def get_main_window_size(self):
        return self._main_window_size

    main_window_size = property(get_main_window_size, set_main_window_size, "")

    def set_offline(self, mode):
        if self._offline != mode:
            self._offline = mode
            self.client.set_bool(GCONF_STRAW_OFFLINE, mode)
            self.emit_signal(straw.OfflineModeChangedSignal(self))
        return

    def get_offline(self):
        return self._offline

    offline = property(get_offline, set_offline, None, "")

    def set_main_pane_position(self, position):
        if self._main_pane_position != position:
            self._main_pane_position = position
            self.client.set_int(GCONF_STRAW_MAIN_PANE_POSITION, position)
            self.emit_signal(straw.MainPanePositionChangedSignal(self))

    def get_main_pane_position(self):
        return self._main_pane_position

    main_pane_position = property(get_main_pane_position, set_main_pane_position, None, "")

    def set_sub_pane_position(self, position):
        if self._sub_pane_position != position:
            self._sub_pane_position = position
            self.client.set_int(GCONF_STRAW_SUB_PANE_POSITION, position)
            self.emit_signal(straw.SubPanePositionChangedSignal(self))

    def get_sub_pane_position(self):
        return self._sub_pane_position

    sub_pane_position = property(get_sub_pane_position, set_sub_pane_position, None, "")

