from mx import DateTime
from mx.DateTime.Parser import DateTimeFromString
import xml.sax.saxutils as XSS
import xml.sax.xmlreader as XSX
import HTMLParser
import straw, error
import re, string, StringIO, cgi

# let's try to be a bit forgiving about the HTML contained in description.
# like allow unquoted attributes containing commas
HTMLParser.HTMLParser.attrfind = re.compile(
    r'\s*([a-zA-Z_][-.:a-zA-Z_0-9]*)(\s*=\s*'
    r'(\'[^\']*\'|"[^"]*"|[-a-zA-Z0-9.,/:;+*%?!&$\(\)_#=~]*))?')

class SXMLGenerator(XSS.XMLGenerator):
    def direct_characters(self, chars):
        self._out.write(chars)

class ItemDescriptionHandler(HTMLParser.HTMLParser):
    def __init__(self, feed):
        HTMLParser.HTMLParser.__init__(self)
        self._images = []
        self._feed = feed
        self._outstream = StringIO.StringIO()
        self._xg = SXMLGenerator(self._outstream, encoding='utf-8')

    def handle_starttag(self, tag, attrs):
        def t2d(t):
            d = {}
            for i in t:
                d[i[0]] = i[1]
            return d
        if tag == 'img':
            attrs = filter(lambda pair: pair[0] == 'src', attrs)
            if len(attrs) > 0:
                url = attrs[0][1]
                curl = self._feed.complete_url(url)
                self._images.append(curl)
        if tag != 'font':
            self._xg.startElement(tag, XSX.AttributesImpl(t2d(attrs)))
        return

    def handle_endtag(self, tag):
        if tag != 'font':
            self._xg.endElement(tag)

    def handle_data(self, data):
        self._xg.characters(data)

    def handle_charref(self, ref):
        text = "&#%s;" % ref
        self._xg.direct_characters(text)

    def handle_entityref(self, ref):
        text = "&%s;" % ref
        self._xg.direct_characters(text)

    def get_images(self):
        return self._images

    def get_data(self):
        return self._outstream.getvalue()

iso_date_re = re.compile('^\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d')
def unicode_field(dict, key, enc='utf-8', default=''):
    v = dict.get(key, default)
    if v != '':
        try:
            v = unicode(v, enc)
        except (ValueError, UnicodeError):
            # try with iso-8859-1, usually we get at least something
            enc = "iso-8859-1"
            v = unicode(v, enc).strip()
    return v

def read_date(s):
    s = s.strip()
    try:
        if iso_date_re.match(s):
            d = DateTimeFromString(s, ('iso',))
        else:
            d = DateTimeFromString(s, ('lit',))
    except ValueError:
        d = DateTime.now()
    return d


#  encoding regex
xmlheaderRe = re.compile('<\?.*encoding="(.*?)".*\?>')

class SummaryParser:
    def __init__(self):
        pass

    def parse(self, content, feed):
        parser = straw.FeedParser()
        parser.feed(content)
        parsed = straw.ParsedSummary()

        enc = parser.channel.get('encoding', '').lower()

        if enc == '':
            match = xmlheaderRe.match(content)
            if match:
               enc =  parser.channel['encoding'] = match.groups()[0].lower()

        parsed.title = unicode_field(parser.channel, "title", enc)
        parsed.description = unicode_field(parser.channel, "description", enc)
        parsed.link = unicode_field(parser.channel, "link", enc)
        parsed.copyright = unicode_field(parser.channel, "rights", enc)
        parsed.last_build_date = read_date(parser.channel.get("date", ""))

        creator = unicode_field(parser.channel, "creator", enc)
        if creator is not None and (len(creator) > 0):
            parsed.creator = creator

        # blogchannel
        blogroll = unicode_field(parser.channel, "blogRoll", enc)
        if blogroll is not None:
            parsed.blogroll = blogroll
        subscriptions = unicode_field(parser.channel, "mySubscriptions", enc)
        if subscriptions is not None:
            parsed.subscriptions = subscriptions
        blink = unicode_field(parser.channel, "blink", enc)
        if blink is not None:
            parsed.blink = blink

        # item properties
        for idict in parser.items:
            item = straw.SummaryItem()
            item.feed = feed
            item.title = unicode_field(idict, 'title', enc)
            item.link = idict.get('link', '')
            if item.link != '':
                item.link = item.link.strip()

            item.description = unicode_field(idict, 'description', enc)

            # dc:creator
            if idict.has_key('creator'):
                item.creator = unicode_field(idict, 'creator', enc)

            # content_encoded
            if idict.has_key('content_encoded'):
                ce = unicode_field(idict, 'content_encoded', enc)
                if ce is not None:
                    item.description = ce

            item.guid = unicode_field(idict, 'guid', enc)
            item.pub_date = read_date(idict.get('date', ""))

            if idict.has_key('source'):
                st = unicode_field(idict, 'source', enc)
                # source format: "url,name"   e.g. 'http://blah.foo,BlahFoo'
                # .. the first comma is all that matters
                url, text = string.split(st, ',', 1 )
                item.source = {'url': url.strip(), 'text': text}

            if item.description is not None:
                idh = ItemDescriptionHandler(feed)
                try:
                    idh.feed(item.description)
                    images = idh.get_images()
                    for i in images:
                        item.add_image(i)
                    item.description = idh.get_data()
                except Exception, ex:
                    error.log(_("Exception Occurred: %s") % ex)
                    item.feed.error = _("Error parsing item %s: %s") % (item.title, ex)

            if idict.has_key('license'):
                # freshmeat
                if parser.namespacemap.has_key('fm'):
                    item.fm_license = unicode_field(idict, 'license', enc)
                    item.fm_changes = unicode_field(idict, 'changes', enc)
                # assume it's creativeCommons or other 'license'
                else:
                    license = unicode_field(idict, "license", enc)
                    item.license_urls.append(license)


            # prism
            if idict.has_key('publicationName'):
                item.publication_name = unicode_field(idict,'publicationName', enc)
                item.publication_volume = unicode_field(idict, 'volume', enc)
                item.publication_number = unicode_field(idict, 'number', enc)
                item.publication_section = unicode_field(idict, 'section', enc)
                item.publication_starting_page = unicode_field(idict, 'startingPage', enc)

            parsed.addItem(item)

        parser.reset()

        return parsed

